#!/usr/bin/perl
#################################################################################################
#               Apache
#################################################################################################
# This script reads access.log file for apache
#
# Author: Tom Webb
# Version : 0.1
# Date : 22-Jul-2010
#
# Copyright 2010 Tom Webb
# Based off of IIS Plugin
#  This file is part of log2timeline.
#
#    log2timeline is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    log2timeline is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with log2timeline.  If not, see <http://www.gnu.org/licenses/>.

package Log2t::input::apache;
use Log2t::Common ':binary';
use Log2t::BinRead;
use HTTP::Date;
use strict;
my $line;
my $VERSION;

$VERSION = '0.1';


#       get_version
# A simple subroutine that returns the version number of the format file
#
# @return A version number

sub get_version()
{
    return "\t" . $VERSION;
}


#       get_description
# A simple subroutine that returns a string containing a description of
# the funcionality of the format file. This string is used when a list of
# all available format files is printed out
#
# @return A string containing a description of the format file's functionality
sub get_description()
{
       return "Parse content of an apache access log file";
}


sub prepare_file()
{

   my $return = 1;
       my $class = shift;
       my $file = shift;

   open( FILE, $file) or  $return =0;     
   return $return;;

}

#       load_line
# A subroutine that reads a line from the access file and returns it to the
# main script
# @return A string containing one line of the log file (or a -1 if we've reached
#       the end of the log file)
sub load_line()
{
       if( $line = <FILE> )
       {
               return 1;
       }
       else
       {
               return 0;
       }

}


#       close_file
# A subroutine that closes the file, after it has been parsed
# @return An integer indicating that the close operation was successful
sub close_file()
{
       close( FILE );  
       return 1;
}



#       parse_line
# This is the main "juice" of the format file.  It takes a line from the log file
# and parses it to produce an array containing all the needed values to print a
# body file.
#
# @param LINE a string containing a single line from the access file
# @return Returns a array containing the needed values to print a body file
#regex used for log was from http://www.cs.cf.ac.uk/Dave/PERL/node241.html
#http://httpd.apache.org/docs/1.3/logs.html#accesslog
#127.0.0.1 - frank [10/Oct/2000:13:55:36 -0700] "GET /apache_pb.gif HTTP/1.0" 200 2326 "http://www.example.com/start.html" "Mozilla/4.08 [en] (Win98; I ;Nav)"

sub parse_line()
{
	use HTTP::Date;

	my %t_line;
	my $text;
	my $w;
	my $w = "(.+?)";
	$line =~ m/^$w $w $w \[$w:$w $w\] "$w $w $w" $w $w "$w" "$w"/;

	my $clientaddress = $1;
	my $hyphen  = $2;
	my $username = $3;
	my $date = $4;
	my $time = $5;
	my $gmt = $6;
	my $method = $7;
	my $httprequest = $8;
	my $proto = $9;
	my $status = $10;
	my $size = $11;
	my $referer = $12;
	my $agent = $13;
	my $datetime = "$date:$time";
	
	my $epoc = str2time( $datetime, "$gmt" ); #use HTTP::Date to conver to epoc it will use the timezone specied in the log.

	%t_line = (
               'time' => { 0 => { 'value' => $epoc, 'type' => 'Entry written', 'legacy' => 15 } },
               'desc' => $text,
               'short' => 'URL: ' . $httprequest,
               'source' => 'Apache',
               'sourcetype' => 'Apache Access.log',
        	'version' => 2,
               'extra' => { 'user' => $username, 'src-ip' => $clientaddress, 'method' => $method, 'prot version' => $proto, 'size' => $size, 'referrer' => $referer, 'agent' => $agent }
       );

       return \%t_line;
}
       
#       get_help
# A simple subroutine that returns a string containing the help
# message for this particular format file.
# @return A string containing a help file for this format file
sub get_help()
{
       return "This parser parses the Apache Access.log log file. To see the definition of the
log format, please see:
http://httpd.apache.org/docs/1.3/logs.html#accesslog
Use with the FILE option as the Access log file\n
\t$0 -f apache access.log

This format file depends upon the library
       HTTP::Date
for converting date variables to epoch time. Possible to install using
perl -MCPAN -e shell
(when loaded)
install HTTP::Date\n";

}

#    verify
# A subroutine that reads a single line from the log file and verifies that it is of the
# correct format so it can be further processed.
# The correct format of a Apache access file is:
# IP - Username date time Method  URI Status size Referer Useragent
# @return An array containing an integer and a string.  The integer indicates a success or failure and the
#    string is the error message (if the file is not correctly formed)
sub verify($$)
{
my $class = shift;
       my $file = shift;       # a reference to the file name
       my $detail = shift;     # use a quick check or skip it


       # define an array to keep
       my %return;
       my $vline;

       $return{'success'} = 1;

 return \%return;
}

1;
